Package: isoband
Title: Generate Isolines and Isobands from Regularly Spaced Elevation
        Grids
Version: 0.3.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Claus O.", "Wilke", , "wilke@austin.utexas.edu", role = "aut",
           comment = c("Original author", ORCID = "0000-0002-7470-9261")),
    person("Thomas Lin", "Pedersen", , "thomas.pedersen@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5147-4711")),
    person("Posit, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: A fast C++ implementation to generate contour lines
    (isolines) and contour polygons (isobands) from regularly spaced grids
    containing elevation data.
License: MIT + file LICENSE
URL: https://isoband.r-lib.org, https://github.com/r-lib/isoband
BugReports: https://github.com/r-lib/isoband/issues
Imports: cli, grid, rlang, utils
Suggests: covr, ggplot2, knitr, magick, bench, rmarkdown, sf, testthat
        (>= 3.0.0), xml2
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-12-05
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/build/compilation-database: true
LinkingTo: cpp11
NeedsCompilation: yes
Packaged: 2025-12-05 12:52:07 UTC; thomas
Author: Hadley Wickham [aut] (ORCID: <https://orcid.org/0000-0003-4757-117X>),
  Claus O. Wilke [aut] (Original author, ORCID:
    <https://orcid.org/0000-0002-7470-9261>),
  Thomas Lin Pedersen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>),
  Posit, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Thomas Lin Pedersen <thomas.pedersen@posit.co>
Repository: CRAN
Date/Publication: 2025-12-07 06:10:07 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 05:37:32 UTC; unix
Archs: isoband.so.dSYM
